/*-------------------------------------------------------- 
 * Module Name : transport manager api
 * Version : 1.4.1
 * 
 * Software Name : Transport Manager
 * Version : 1.4.1
 *
 * Copyright © 01-10-2012 – 01-10-2012 France Télécom
 * This software is distributed under the Apache (v2) license,
 * the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html.
 * or see the "license.txt" file for more details.
 *--------------------------------------------------------
 * File Name   : ${file_name}
 *
 * Created     : ${date}
 * Author(s)   : ${authors}
 *
 * Description :
 * Main interface of transport manager. (may be connected through camel for instance)
 *
 *--------------------------------------------------------
 * ${Log}
 *
 */
package com.orange.labs.quickpass.tm.service;

import com.orange.labs.quickpass.enums.TransportBearer;
import com.orange.labs.quickpass.tm.service.cattp.AlreadyConnectedException;
import com.orange.labs.quickpass.tm.service.cattp.CattpConnectionInfo;
import com.orange.labs.quickpass.tm.service.cattp.NoMorePortAvailableException;
import com.orange.labs.quickpass.tm.service.cattp.SessionNotFoundException;
import com.orange.labs.quickpass.type.Msisdn;
import java.util.List;

/**
 * Main interface of transport manager. (may be connected through camel for instance)
 *
 * @author Gael Gerard
 *
 */
public interface TransportService {

    /**
     * Open a cattp connection and get an available cattp port
     *
     * @param msisdn msisdn
     * @return CattpConnectionInfo : Valid cattp Port value when available, null or exception otherwise.
     * @throws NoMorePortAvailableException : All the cattp port are busy
     * @throws AlreadyConnectedException : the cattp session is already in a connected state
     */
    CattpConnectionInfo openCattpLink(Msisdn msisdn)
            throws NoMorePortAvailableException, AlreadyConnectedException;

    /**
     * Sends an SMS text to the targeted msisdn
     *
     * @param msisdn targeted msisdn or msisdn list
     * @param message SMS text message
     */
    void sendSMSText(Msisdn msisdn[], String message);

    /**
     * Sends a wap push SMS to the targeted MSISDN
     *
     * @param msisdn targeted msisdn or msisdn list
     * @param message message to display on push
     * @param url url to trigger on browser
     */
    void sendWAPPushSMS(Msisdn msisdn[], String message, String url);

    /**
     * Sends a Push Sms on a given port
     *
     * @param msisdn targeted msisdn or msisdn list
     * @param message Associated message may be null
     * @param port port where SMS will be pushed
     */
    void sendPushSMS(Msisdn msisdn[], String message, String port);

    /**
     * Sends OTA secure packets either using SMS or CATTP <BR/> Before sending data on CATTP Bearer, the channel shall be opened.
     * It can be done through a SMS Push BIP (Open Channel) see ETSI 102.223 <BR/>
     *
     * For instance, an openchannel (BIP) command can be (in hexadecimal) : <BR/> 80EC0101443507020000030000023C030121DA <BR/>
     * 4707066F72616E6765<BR/>390203E00D0F08FEFF<BR/>006F00720061006E00670065<BR/>0D0F08FEFF
     * <BR/>006F00720061006E00670065<BR/>3E05217F000001 <BR/> and an openchannel (CAT) command : <BR/> 80EC0102053C030101C2 <BR/>
     *
     * After this push bip SMS, secured packets can be send with CATTP bearer
     *
     * To summarize for a cattp session, the process below shall be followed :<BR/> - send a SMS push BIP (SMS bearer) <BR/> -
     * send secured packets (CATTP bearer)
     *
     * @param securePackets Secure packets array (see ETSI 102.225)
     * @param msisdn targeted msisdn
     * @param bearer ota bearer (SMS or CATTP)
     * @throws SessionNotFoundException : no session found for cattp bearer
     */
    void sendSecurePackets(Msisdn msisdn, List<byte[]> securePackets,
            TransportBearer bearer) throws SessionNotFoundException;

    /**
     * Closes cattp the connection
     *
     * @param msisdn msisdn
     * @throws SessionNotFoundException : session already closed
     */
    void closeCattpLink(Msisdn msisdn) throws SessionNotFoundException;

     /**
     * Get the status for a cattp session 
     * @param msisdn targeted msisdn
     * @return status {CREATED,OPENED,CLOSED}
     * @throws SessionNotFoundException when no session has been found
     */
    CattpSessionStatus getCattpSessionStatus(Msisdn msisdn) throws SessionNotFoundException;
}
